// Browser.cpp : Implementation of CBrowser
// Written by Alex Ivanov (consul@aleksoft.net)
// Copyright (C) 2004
//
// If you find this program useful, you may chose to send a small 
// contibution via paypal to consl@aleksoft.net, 
// though no obligations are implied.

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, version 2 of the License only.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "stdafx.h"
#include "AleksoftNavigator.h"
#include "Browser.h"

WNDPROC oldProc=0; //original WindowProc of Internet Explorer
/////////////////////////////////////////////////////////////////////////////
// CBrowser

//Handling events from the DialogBox

BOOL CALLBACK CBrowser::DlgProc(
	HWND hwndDlg,  // handle to dialog box
	UINT uMsg,     // message
	WPARAM wParam, // first message parameter
	LPARAM lParam  // second message parameter
	)
{
	if (uMsg==WM_COMMAND)//Command button pressed
	{
		CBrowser* CB=(CBrowser*)statCB;
		if (LOWORD(wParam)==IDC_BUTTON1) //Back
		{
			CB->GotoPrev();
		}
			
		if (LOWORD(wParam)==IDC_BUTTON2) //Close
		{
			SetWindowLongW(CB->hWndApp,GWL_WNDPROC,(long)oldProc);//Restore original WndProc
			CB->hDlg=0;
			CB=0;
			DestroyWindow(hwndDlg);
			oldProc=0;
		}
		if (LOWORD(wParam)==IDC_BUTTON3) //Next
			CB->GotoNext();
		return 1;
	}
	else if(uMsg==WM_USER+1)
	{
		//MoveWindow(hwndDlg,wParam,25,100,100,0);
		SetWindowPos(hwndDlg,HWND_TOP,wParam,25,0,0, SWP_NOSIZE);
	}
	return 0;
}

//Catch Resize events form IE
LRESULT CALLBACK CBrowser::WindowProc(
  HWND hwnd,      // handle to window
  UINT uMsg,      // message identifier
  WPARAM wParam,  // first message parameter
  LPARAM lParam   // second message parameter
)
{
	//LRESULT retval=CallWindowProc(oldProc,hwnd,uMsg,wParam,lParam);
	if(uMsg==WM_SIZE)
	{
		RECT rect;
		CBrowser* CB=(CBrowser*)statCB;
		GetClientRect(CB->hDlg,&rect);
		long left;
		CB->m_pWb->get_Width(&left);
		left-=(rect.right-rect.left+10);
		PostMessage(CB->hDlg,WM_USER+1,left,0);
	}
	return CallWindowProc(oldProc,hwnd,uMsg,wParam,lParam);
}

void CBrowser::GotoPrev()
{
	int n=255; //Length of the string buffer
	fseek(m_pFile,0,SEEK_SET); //Reset File pointer 
	for(int i=0;i<m_pCurrentPos;i++) //Skip records
		if(fgetws(m_pBuffer,n,m_pFile))
		{
			HWND hw=GetDlgItem(hDlg,IDC_BUTTON3);//Enable Next button
			EnableWindow(hw,TRUE);
			m_pBuffer[n-1]=0;//Clear CR character
		}
	if(m_pCurrentPos<1)
		{
			HWND hw=GetDlgItem(hDlg,IDC_BUTTON1); //Disable Prev button
			EnableWindow(hw,FALSE);
		}
	m_pCurrentPos--; //Decrement the counter
	GoToSite(m_pBuffer);
}

void CBrowser::GotoNext()
{
	int n=255;
	if(fgetws(m_pBuffer,n,m_pFile))
	{
		HWND hw=GetDlgItem(hDlg,IDC_BUTTON1);
		EnableWindow(hw,TRUE);
		m_pBuffer[n-1]=0;//Clear CR
		GoToSite(m_pBuffer);
		m_pCurrentPos++;
	}
	else
	{
		HWND hw=GetDlgItem(hDlg,IDC_BUTTON3);
		EnableWindow(hw,FALSE);
	}
}

CBrowser::CBrowser() //initilaize vars
{
	hDlg=0;
	m_pCurrentPos=0;
	oldProc=0;
}

HRESULT CBrowser::FinalConstruct()
{
	//Open the urls file
	//char buff[255];
	m_pBuffer=new unsigned short[255];
	HKEY key;
	ULONG type;
	ULONG len=255;
	RegOpenKeyEx(HKEY_LOCAL_MACHINE,"SOFTWARE\\Microsoft\\Internet Explorer\\Extensions\\{8ed8efc7-3a23-46af-9e16-4e8000027a9e}",0,KEY_QUERY_VALUE,&key);
	if(RegQueryValueEx(key,"DataFile",0,&type,(unsigned char*)m_pBuffer,&len)==ERROR_SUCCESS)
	{
		if(m_pFile=fopen((const char*)m_pBuffer,"r"))
			return S_OK;
		else 
			return E_INVALIDARG;
	}
	else
		return E_INVALIDARG;
}

void CBrowser::GoToSite(BSTR bstrUrl)
{
	VARIANT url, v[4]={0,0,0,0};//Create null VARIANTs
	BSTR b=SysAllocString(bstrUrl);//Convert URL string to VARIANT
	V_VT(&url)=VT_BSTR;
	V_BSTR(&url)=b;
	m_pWb->Navigate2(&url,v,v+1,v+2,v+3);//Direct the Browser to the site
	if(b) SysFreeString(b); //Release memory
}


STDMETHODIMP CBrowser::SetSite(IUnknown *pSite)
{
	IServiceProvider* isp;
	if(!pSite) //Browser is shutting down!
	{ 
		DestroyWindow(hDlg);
		m_pSite=0;
		//m_pWb->Release();
		m_pWb=0;
		return S_OK;//Quitting
	}
	m_pSite=pSite; //pSite valid. Initializing...
	HRESULT hr=pSite->QueryInterface(IID_IServiceProvider,(void**)&isp);
	if(hr!=S_OK) 
	{
		MessageBox(0,_T("Can't get provider"),0,0);
		return E_FAIL;
	}
	//isp->AddRef();
	hr=isp->QueryService(IID_IWebBrowserApp,&m_pWb);
	if(hr!=S_OK) 
	{
		MessageBox(0,_T("Can't get browser"),0,0);
		return E_FAIL;
	}
	//isp->Release();
	isp=0; //Don't need the ServiceProvider any longer
	
	//DlgProc can't access the class non-static members directly
	// so we're assigning the instance of the class to the static pointer
	statCB=(long)this;
	//ShowDlg();
	GotoNext(); //Go directly to the first site on the list
	return S_OK;
}

STDMETHODIMP CBrowser::GetSite(REFIID priid, void *ppvObj)
{
	m_pSite->QueryInterface(priid,&ppvObj); //inform the browser that we're ok?
	return S_OK;
}

STDMETHODIMP CBrowser::QueryStatus(const _GUID* pGuid,ULONG cCmds,_tagOLECMD* prgCmd,OLECMDTEXT* pCmdTxt)
{
	pGuid=0;
	for(ULONG i=0;i<cCmds;i++)
		//Tell the browser that we support the comand button
		// or the browser will disable it after first use.
		prgCmd[i].cmdf=OLECMDF_SUPPORTED | OLECMDF_ENABLED; 
	return S_OK;
}

STDMETHODIMP CBrowser::Exec(const GUID* pGuid,DWORD nCmdID,DWORD nCmdExecOpt,VARIANT* pvaIn,VARIANT* pvaOut)
{
	ShowDlg(); //Put our dialog box on the IE's command bar
	return S_OK;
}

void CBrowser::ShowDlg()
{
	if(!hDlg)
	{
		HRESULT hr=m_pWb->get_HWND((long*)&hWndApp);
		HWND hw=FindWindowEx(hWndApp,0,_T("WorkerW"),0);
		if(hw)
			;
		else
		{
			TCHAR* buff;
			DWORD err;
			err=GetLastError();
			err=FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,0, err,0,(LPTSTR)&buff,0,0);
			MessageBox(0,buff,0,0);
			LocalFree(buff);
		}
		hDlg=CreateDialog(_Module.m_hInst,MAKEINTRESOURCE(IDD_FORMVIEW),hw,static_cast<DLGPROC>(DlgProc));
		SetParent(hDlg,hw);
		RECT rect;
		GetClientRect(hDlg,&rect);
		long left;
		m_pWb->get_Width(&left);
		left-=(rect.right-rect.left+10);

		if(!MoveWindow(hDlg,left,25,100,100,0))
		{
			TCHAR* buff;
			DWORD err;
			err=GetLastError();
			err=FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,0, err,0,(LPTSTR)&buff,0,0);
			MessageBox(0,buff,0,0);
			LocalFree(buff);
		}
		if (!oldProc)
			oldProc=(WNDPROC)SetWindowLongW(hWndApp,GWL_WNDPROC,(long)WindowProc);

	}
}